@@ -71,13 +71,13 @@ class Driver {
 				
 				return 0;
 			
-			case "--local-edit":
+			case "--edit":
 				if (i+1 == args.Length) {
-					Console.WriteLine ("--local-edit requires an argument");
+					Console.WriteLine ("Usage: --edit path, where path is to the location of Monodoc-format XML documentation files.");
 					return 1; 
 				}
 				EcmaUncompiledHelpSource hs = new EcmaUncompiledHelpSource(args[i+1]);
-				RootTree.ExtraHelpSources.Add(hs.Name + " (Local Edit)", hs);
+				RootTree.ExtraHelpSources.Add(hs.Name, hs);
 				i++;
 				break;
 				
@@ -109,25 +109,22 @@ class Browser {
 	[Glade.Widget] public Window window1;
 	[Glade.Widget] TreeView reference_tree;
 	[Glade.Widget] TreeView bookmark_tree;
-	[Glade.Widget] ScrolledWindow html_container;
-	[Glade.Widget] Statusbar statusbar;
-	[Glade.Widget] Button back_button, forward_button;
+	[Glade.Widget] public Statusbar statusbar;
+	[Glade.Widget] public Button back_button, forward_button;
 	[Glade.Widget] Entry index_entry;
 	[Glade.Widget] CheckMenuItem editing1;
 	[Glade.Widget] CheckMenuItem showinheritedmembers;
 	[Glade.Widget] CheckMenuItem comments1;
 	[Glade.Widget] MenuItem postcomment;
-	[Glade.Widget] MenuItem paste1;
+	[Glade.Widget] public MenuItem paste1;
+	public Notebook tabs_nb;
+	public Tab CurrentTab;
+	bool HoldCtrl;
 
 	[Glade.Widget] MenuItem bookmarksMenu;
 
-	//
-	// Editor
-	//
-	[Glade.Widget] Notebook html_and_editor_notebook;
-	[Glade.Widget] TextView text_editor;
-	[Glade.Widget] ScrolledWindow html_preview_container;
-
+	[Glade.Widget] VBox help_container;
+	
 	[Glade.Widget] EventBox bar_eb, index_eb;
 	[Glade.Widget] Label subtitle_label;
 	[Glade.Widget] Notebook nb;
@@ -143,14 +140,6 @@ class Browser {
 	
 	Gdk.Pixbuf monodoc_pixbuf;
 
-	public History history;
-
-	// Where we render the contents
-	HTML html;
-
-	// Our HTML preview during editing.
-	HTML html_preview;
-
         //
 	// Left-hand side Browsers
 	//
@@ -161,7 +150,7 @@ class Browser {
 	internal RootTree help_tree;
 
 	// For the status bar.
-	uint context_id;
+	public uint context_id;
 
 	// Control of Bookmark
 	struct BookLink
@@ -186,6 +175,7 @@ class Browser {
 		MainWindow.DeleteEvent += new DeleteEventHandler (delete_event_cb);
                 
 		MainWindow.KeyPressEvent += new KeyPressEventHandler (keypress_event_cb);
+		MainWindow.KeyReleaseEvent += new KeyReleaseEventHandler (keyrelease_event_cb);
                 
 		Stream icon = GetResourceImage ("monodoc.png");
 
@@ -203,7 +193,7 @@ class Browser {
 		title_label.Layout.FontDescription = fd;
 		title_label_box.Add (title_label);
 		title_label.Show ();
-
+		
 		//colour the bar according to the current style
 		bar_style = bar_eb.Style.Copy ();
 		bar_eb.Style = bar_style;
@@ -213,79 +203,71 @@ class Browser {
 		help_tree = RootTree.LoadTree ();
 		tree_browser = new TreeBrowser (help_tree, reference_tree, this);
 		
+		//
+		// Tab Notebook and first tab
+		//
+		tabs_nb = new Notebook(); //the Notebook that holds tabs
+		tabs_nb.Scrollable = true;
+		tabs_nb.SwitchPage += new SwitchPageHandler(ChangeTab);
+		help_container.Add(tabs_nb);
+
+		
 		// restore the editing setting
 		editing1.Active = SettingsHandler.Settings.EnableEditing;
 
 		comments1.Active = SettingsHandler.Settings.ShowComments;
 
-		//
-		// Setup the HTML rendering area
-		//
-		html = new HTML ();
-		html.Show ();
-		html_container.Add (html);
-                html.LinkClicked += new LinkClickedHandler (LinkClicked);
-		html.OnUrl += new OnUrlHandler (OnUrlMouseOver);
-		html.UrlRequested += new UrlRequestedHandler (UrlRequested);
-		context_id = statusbar.GetContextId ("");
-
-		//
-		// Text editor (for editing the documentation).
-		//
-		html_and_editor_notebook.ShowTabs = false;
-		html_preview = new HTML ();
-		html_preview.Show ();
-		html_preview_container.Add (html_preview);
-
-		text_editor.Buffer.Changed += new EventHandler (EditedTextChanged);
-		text_editor.WrapMode = WrapMode.Word;
-		
 		paste1.Sensitive = false;
 
 		//
 		// Other bits
 		//
-		history = new History (back_button, forward_button);
 		bookList = new ArrayList ();
 
 		index_browser = IndexBrowser.MakeIndexBrowser (this);
 		
+		AddTab();
+		MainWindow.ShowAll();
+	}
+
+	// Adds a Tab and Activates it
+	void AddTab() 
+	{
+		CurrentTab = new Tab (this);
+		tabs_nb.AppendPage (CurrentTab, CurrentTab.TabLabel);
+		tabs_nb.ShowTabs = (tabs_nb.NPages > 1);
+		tabs_nb.ShowAll (); //Needed to show the new tab
+		tabs_nb.CurrentPage = tabs_nb.PageNum (CurrentTab);
+		//Show root node
 		Node match;
 		string s = help_tree.RenderUrl ("root:", out match);
 		if (s != null){
 			Render (s, match, "root:");
-			history.AppendHistory (new Browser.LinkPageVisit (this, "root:"));
+			CurrentTab.history.AppendHistory (new Browser.LinkPageVisit (this, "root:"));
 		}
-	}
-
-	public enum Mode {
-		Viewer, Editor
-	}
-
-	public Mode BrowserMode {
-		get {
-			return browser_mode;
-		}
-		set {
-			browser_mode = value;
-		}
-	}
-
-	Mode browser_mode;
-	public void SetMode (Mode m)
-	{
-		if (browser_mode == m)
-			return;
 		
-		if (m == Mode.Viewer) {
-			html_and_editor_notebook.Page = 0;
+	}
+	
+	//Called when the user changes the active Tab
+	void ChangeTab(object o, SwitchPageArgs args) 
+	{
+		
+		//Deactivate the old history
+		CurrentTab.history.Active = false;
+		
+		//Get the new Tab		
+		CurrentTab = (Tab) tabs_nb.GetNthPage ((int) args.PageNum);
+		title_label.Text = CurrentTab.Title;
+		
+		//Activate the new history
+		CurrentTab.history.Active = true;
+		
+		if (CurrentTab.Tab_mode == Mode.Viewer) {
+			CurrentTab.history.ActivateCurrent();
 			paste1.Sensitive = false;
 		} else {
-			html_and_editor_notebook.Page = 1;
 			paste1.Sensitive = true;
 		}
-
-		browser_mode = m;
 	}
 
 	void BarStyleSet (object obj, StyleSetArgs args)
@@ -301,7 +283,7 @@ class Browser {
 		return s;
 	}
 
-	void UrlRequested (object sender, UrlRequestedArgs args)
+	public void UrlRequested (object sender, UrlRequestedArgs args)
 	{
 		Console.WriteLine ("Image requested: " + args.Url);
 		Stream s = help_tree.GetImage (args.Url);
@@ -336,8 +318,11 @@ class Browser {
 		}
 	}
 	
-	void LinkClicked (object o, LinkClickedArgs args)
+	public void LinkClicked (object o, LinkClickedArgs args)
 	{
+		if (HoldCtrl)
+			AddTab();
+
 		LoadUrl (args.Url);
 	}
 
@@ -349,17 +334,17 @@ class Browser {
 		if (url.StartsWith("#"))
 		{
 			// FIXME: This doesn't deal with whether anchor jumps should go in the history
-			html.JumpToAnchor(url.Substring(1));
+			CurrentTab.html.JumpToAnchor(url.Substring(1));
 			return;
 		}
 
 		if (url.StartsWith ("edit:"))
 		{
 			Console.WriteLine ("Node is: " + url);
-                        edit_node = EditingUtils.GetNodeFromUrl (url, help_tree);
-                        edit_url = url;
-			SetMode (Mode.Editor);
-                        text_editor.Buffer.Text = edit_node.InnerXml;
+                        CurrentTab.edit_node = EditingUtils.GetNodeFromUrl (url, help_tree);
+                        CurrentTab.edit_url = url;
+			CurrentTab.SetMode (Mode.Editor);
+                        CurrentTab.text_editor.Buffer.Text = CurrentTab.edit_node.InnerXml;
 			return;
 		}
 		
@@ -369,7 +354,7 @@ class Browser {
 		string res = help_tree.RenderUrl (url, out node);
 		if (res != null){
 			Render (res, node, url);
-			history.AppendHistory (new LinkPageVisit (this, url));
+			CurrentTab.history.AppendHistory (new LinkPageVisit (this, url));
 			return;
 		}
 		
@@ -392,18 +377,39 @@ class Browser {
 	{
 		CurrentUrl = url;
 
-		Gtk.HTMLStream stream = html.Begin ("text/html");
+		Gtk.HTMLStream stream = CurrentTab.html.Begin ("text/html");
 
 		stream.Write ("<html><body>");
 		stream.Write (text);
 		stream.Write ("</body></html>");
-		html.End (stream, HTMLStreamStatus.Ok);
+		CurrentTab.html.End (stream, HTMLStreamStatus.Ok);
 		if (matched_node != null) {
 			if (tree_browser.SelectedNode != matched_node)
 				tree_browser.ShowNode (matched_node);
-
 			title_label.Text = matched_node.Caption;
-
+		
+			//
+			//Try to find a better name for the Tab
+			//
+			string tabTitle;
+			tabTitle = matched_node.Caption; //Normal title
+			string[] parts = matched_node.URL.Split('/', '#');			
+			if(matched_node.URL != null && matched_node.URL.StartsWith("ecma:")) {
+				if(parts.Length == 3 && parts[2] != String.Empty) { //List of Members, properties, events, ...
+					tabTitle = parts[1] + ": " + matched_node.Caption;
+				} else if(parts.Length >= 4) { //Showing a concrete Member, property, ...					
+						tabTitle = parts[1] + "." + matched_node.Caption;
+				} else {
+					tabTitle = matched_node.Caption;
+				}
+			}
+			//Trim too large titles
+			if(tabTitle.Length > 35) {
+				CurrentTab.Title = tabTitle.Substring(0,35) + " ...";
+			} else {
+				CurrentTab.Title = tabTitle;
+			}
+				
 			if (matched_node.Nodes != null) {
 				int count = matched_node.Nodes.Count;
 				string term;
@@ -426,7 +432,7 @@ class Browser {
 	// Invoked when the mouse is over a link
 	//
 	string last_url = "";
-	void OnUrlMouseOver (object o, OnUrlArgs args)
+	public void OnUrlMouseOver (object o, OnUrlArgs args)
 	{
 		string new_url = args.Url;
 
@@ -446,15 +452,37 @@ class Browser {
 		case Gdk.Key.Left:
 			if (((Gdk.ModifierType) args.Event.State &
 			Gdk.ModifierType.Mod1Mask) !=0)
-			history.BackClicked (this, EventArgs.Empty);
+			CurrentTab.history.BackClicked (this, EventArgs.Empty);
 			args.RetVal = true;
 			break;
 		case Gdk.Key.Right:
 			if (((Gdk.ModifierType) args.Event.State &
 			Gdk.ModifierType.Mod1Mask) !=0)
-			history.ForwardClicked (this, EventArgs.Empty);
+			CurrentTab.history.ForwardClicked (this, EventArgs.Empty);
 			args.RetVal = true;
 			break;
+		case Gdk.Key.Control_L:
+		case Gdk.Key.Control_R:
+			HoldCtrl = true;
+			break;
+		case Gdk.Key.Page_Up:
+			if (HoldCtrl)
+				tabs_nb.PrevPage();
+			break;
+		case Gdk.Key.Page_Down:
+			if (HoldCtrl)
+				tabs_nb.NextPage();
+			break;
+		}
+	}
+	
+	void keyrelease_event_cb (object o, KeyReleaseEventArgs args)
+	{
+		switch (args.Event.Key) {
+		case Gdk.Key.Control_L:
+		case Gdk.Key.Control_R:
+			HoldCtrl = false;
+			break;
 		}
 	}
 	
@@ -470,15 +498,15 @@ class Browser {
 		// postcomment.Sensitive = comments1.Active;
 
 		// refresh, so we can see the comments
-		if (history != null) // catch the case when we are currently loading
-			history.ActivateCurrent ();
+		if (CurrentTab.history != null) // catch the case when we are currently loading
+			CurrentTab.history.ActivateCurrent ();
 	}
 	
 	void OnInheritedMembersActivate (object o, EventArgs args)
 	{
 		SettingsHandler.Settings.ShowInheritedMembers = showinheritedmembers.Active;
-		if (history != null) // catch the case when we are currently loading
-			history.ActivateCurrent ();
+		if (CurrentTab.history != null) // catch the case when we are currently loading
+			CurrentTab.history.ActivateCurrent ();
 	}
 
 	void OnEditingActivate (object o, EventArgs args)
@@ -486,8 +514,8 @@ class Browser {
 		SettingsHandler.Settings.EnableEditing = editing1.Active;
 
 		// refresh, so we can see the [edit] things
-		if (history != null) // catch the case when we are currently loading
-			history.ActivateCurrent ();
+		if (CurrentTab != null) // catch the case when we are currently loading
+			CurrentTab.history.ActivateCurrent ();
 	}
 	
 	void OnCollapseActivate (object o, EventArgs args)
@@ -573,11 +601,11 @@ class Browser {
 	//
 	void OnCopyActivate (object sender, EventArgs a)
 	{
-		if (BrowserMode == Mode.Viewer)
-			html.Copy ();
+		if (CurrentTab.Tab_mode == Mode.Viewer)
+			CurrentTab.html.Copy ();
 		else {
 			Clipboard cb = Clipboard.Get (Gdk.Selection.Clipboard);
-			text_editor.Buffer.CopyClipboard (cb);
+			CurrentTab.text_editor.Buffer.CopyClipboard (cb);
 		}
 	}
 
@@ -593,7 +621,7 @@ class Browser {
 
 		string text = cb.WaitForText ();
 
-		text_editor.Buffer.InsertAtCursor (text);
+		CurrentTab.text_editor.Buffer.InsertAtCursor (text);
 	}
 
 	class About {
@@ -1110,7 +1138,15 @@ class Browser {
 	//
 	void OnSelectAllActivate (object sender, EventArgs a)
 	{
-		html.SelectAll ();
+		CurrentTab.html.SelectAll ();
+	}
+	
+	//
+	// Invoked by New Tab menu entry.
+	//
+	void OnNewTab (object sender, EventArgs a)
+	{
+		AddTab();
 	}
 
 	void BookmarkHandle (object obj, EventArgs args)
@@ -1158,93 +1194,6 @@ class Browser {
 		BookmarkEdit.Show(this);
 	}
 
-	void OnSaveEdits (object sender, EventArgs a)
-	{
-		try {
-			edit_node.InnerXml = text_editor.Buffer.Text;
-		} catch (Exception e) {
-			statusbar.Pop (context_id);
-			statusbar.Push (context_id, e.Message);
-			return;
-		}
-		EditingUtils.SaveChange (edit_url, help_tree, edit_node);
-		SetMode (Mode.Viewer);
-
-		history.ActivateCurrent ();
-	}
-
-	void OnInsertParaClicked (object sender, EventArgs a)
-	{
-		text_editor.Buffer.InsertAtCursor ("\n<para>\n</para>");
-	}
-
-	void OnInsertNoteClicked (object sender, EventArgs a)
-	{
-		text_editor.Buffer.InsertAtCursor ("\n<block subset=\"none\" type=\"note\">\n  <para>\n  </para>\n</block>");
-	}
-
-	void OnInsertExampleClicked (object sender, EventArgs a)
-	{
-		text_editor.Buffer.InsertAtCursor ("\n<example>\n  <code lang=\"C#\">\n  </code>\n</example>");
-	}
-
-	void OnInsertListClicked  (object sender, EventArgs a)
-	{
-		text_editor.Buffer.InsertAtCursor ("\n<list type=\"bullet\">\n  <item>\n    <term>First Item</term>\n  </item>\n</list>");
-
-	}
-
-	void OnInsertTableClicked (object sender, EventArgs a)
-	{
-		text_editor.Buffer.InsertAtCursor ("\n<list type=\"table\">\n  <listheader>\n    <term>Column</term>\n" +
-						   "    <description>Description</description>\n" +
-						   "  </listheader>\n" +
-						   "  <item>\n" +
-						   "    <term>Term</term>\n" +
-						   "    <description>Description</description>\n" +
-						   "  </item>\n" +
-						   "</list>");
-	 }
-
-	void OnInsertType (object sender, EventArgs a)
-	{
-		text_editor.Buffer.InsertAtCursor ("<see cref=\"T:System.Object\"/>");
-	}
-	
-	void OnCancelEdits (object sender, EventArgs a)
-	{
-		SetMode (Mode.Viewer);
-		history.ActivateCurrent ();
-	}
-
-	void EditedTextChanged (object sender, EventArgs args)
-	{
-		StringWriter sw = new StringWriter ();
-		XmlWriter w = new XmlTextWriter (sw);
-		
-		try {
-			w.WriteStartElement ("html");
-			w.WriteStartElement ("body");
-			
-			edit_node.InnerXml = text_editor.Buffer.Text;
-			EditingUtils.RenderEditPreview (edit_url, help_tree, edit_node, w);
-			
-			w.WriteEndElement ();
-			w.WriteEndElement ();
-			
-			w.Flush ();
-		} catch (Exception e) {
-			statusbar.Pop (context_id);
-			statusbar.Push (context_id, e.Message);
-			return;
-		}
-		statusbar.Pop (context_id);
-		statusbar.Push (context_id, "XML OK");
-		Gtk.HTMLStream s = html_preview.Begin ("text/html");
-		s.Write (sw.ToString ());
-		html_preview.End (s, HTMLStreamStatus.Ok);
-	}
-
 	class BookmarkEdit {
 		[Glade.Widget] Window bookmarks_edit;
 		[Glade.Widget] Button bookmarks_delete;
@@ -1532,7 +1481,7 @@ class TreeBrowser {
 	void RowActivated  (object sender, EventArgs a)
 	{
 
-		browser.SetMode (Browser.Mode.Viewer);
+		browser.CurrentTab.SetMode (Mode.Viewer);
 
 		if (IgnoreRowActivated)
 			return;
@@ -1556,7 +1505,7 @@ class TreeBrowser {
 				s = n.tree.HelpSource.GetText (url, out match);
 				if (s != null){
 					((Browser)browser).Render (s, n, url);
-					browser.history.AppendHistory (new NodePageVisit (browser, n, url));
+					browser.CurrentTab.history.AppendHistory (new NodePageVisit (browser, n, url));
 					return;
 				}
 			}
@@ -1567,7 +1516,7 @@ class TreeBrowser {
 			s = help_tree.RenderUrl (url, out match);
 			if (s != null){
 				((Browser)browser).Render (s, n, url);
-				browser.history.AppendHistory (new Browser.LinkPageVisit (browser, url));
+				browser.CurrentTab.history.AppendHistory (new Browser.LinkPageVisit (browser, url));
 				return;
 			}
 
@@ -1849,4 +1798,284 @@ class IndexBrowser {
 		}
 	}
 }
+
+public enum Mode {
+		Viewer, Editor
+	}
+	
+//
+// A Tab is a Notebok with two pages, one for editing and one for visualizing
+//
+class Tab : Notebook {
+	
+	// Our HTML preview during editing.
+	public HTML html_preview;
+	
+	// Where we render the contents
+	public HTML html;
+	
+	public TextView text_editor;
+	public Mode Tab_mode;
+	public History history;
+	private Browser browser;
+	private Label titleLabel;
+	private Image EditImg;
+	public HBox TabLabel;
+	
+	public string Title {
+		get { return titleLabel.Text; }
+		set { titleLabel.Text = value; }
+	}
+	
+	public System.Xml.XmlNode edit_node;
+	public string edit_url;
+	
+	public Tab(Browser br) 
+	{
+
+		browser = br;
+		ShowTabs = false;
+		ShowBorder = false;
+		TabBorder = 0;
+		TabHborder = 0;
+		history = new History (browser.back_button, browser.forward_button);
+		
+		//
+		// First Page
+		//
+		ScrolledWindow html_container = new ScrolledWindow();
+		
+		//
+		// Setup the HTML rendering area
+		//
+		html = new HTML ();
+		html.Show ();
+		html_container.Add (html);
+		html.LinkClicked += new LinkClickedHandler (browser.LinkClicked);
+		html.OnUrl += new OnUrlHandler (browser.OnUrlMouseOver);
+		html.UrlRequested += new UrlRequestedHandler (browser.UrlRequested);
+		browser.context_id = browser.statusbar.GetContextId ("");
+		
+		AppendPage(html_container, new Label("Html"));
+		
+		//
+		// Second Page: editing
+		//
+		VBox vbox1 = new VBox(false, 0);
+		
+		VBox MainPart = new VBox(false, 0);
+		
+		//
+		// TextView for XML code
+		//
+		ScrolledWindow sw = new ScrolledWindow();
+		text_editor = new TextView();
+		text_editor.Buffer.Changed += new EventHandler (EditedTextChanged);
+		text_editor.WrapMode = WrapMode.Word;
+		sw.Add(text_editor);
+		
+		//
+		// XML editing buttons
+		//
+		HBox EdBots = new HBox(false, 2);
+		Button button1 = new Button("<e_xample>");
+		EdBots.PackStart(button1);
+		Button button2 = new Button("<list>");
+		EdBots.PackStart(button2);
+		Button button3 = new Button("<_table>");
+		EdBots.PackStart(button3);
+		Button button4 = new Button("<_see...>");
+		EdBots.PackStart(button4);
+		Button button5 = new Button("<_para>");
+		EdBots.PackStart(button5);
+		Button button6 = new Button("Add Note");
+		EdBots.PackStart(button6);
+		
+		button1.Clicked += new EventHandler (OnInsertExampleClicked);
+		button2.Clicked += new EventHandler (OnInsertListClicked);
+		button3.Clicked += new EventHandler (OnInsertTableClicked);
+		button4.Clicked += new EventHandler (OnInsertType);
+		button5.Clicked += new EventHandler (OnInsertParaClicked);
+		button6.Clicked += new EventHandler (OnInsertNoteClicked);
+		
+		Frame html_preview_frame = new Frame("Preview");
+		ScrolledWindow html_preview_container = new ScrolledWindow();
+		//
+		// code preview panel
+		//
+		html_preview = new HTML ();
+		html_preview.Show ();
+		html_preview_container.Add (html_preview);
+		
+		html_preview_frame.Add(html_preview_container);
+		
+		MainPart.PackStart(sw);
+		MainPart.PackStart(EdBots, false, false, 0);
+		MainPart.PackStart(html_preview_frame);
+		
+		//
+		// Close and Save buttons
+		//
+		HBox MainBots = new HBox(false, 2);
+		HBox Filling = new HBox(false, 0);
+		Button close = new Button("C_lose");
+		Button save = new Button("S_ave");
+		
+		close.Clicked += new EventHandler (OnCancelEdits);
+		save.Clicked += new EventHandler (OnSaveEdits);
+		
+		MainBots.PackStart(Filling);
+		MainBots.PackStart(close, false, false, 0);
+		MainBots.PackStart(save, false, false, 0);
+		
+		vbox1.PackStart(MainPart);
+		vbox1.PackStart(MainBots, false, false, 0);
+		
+		AppendPage(vbox1, new Label("Edit XML"));
+		
+		//
+		//Create the Label for the Tab
+		//
+		TabLabel = new HBox(false, 2);
+		
+		titleLabel = new Label("");
+		
+		//Close Tab button
+		Button tabClose = new Button();
+		Image img = new Image(Stock.Close, IconSize.SmallToolbar);
+		tabClose.Add(img);
+		tabClose.Relief = Gtk.ReliefStyle.None;
+		tabClose.SetSizeRequest (18, 18);
+		tabClose.Clicked += new EventHandler (OnTabClose);
+		
+		//Icon showed when the Tab is in Edit Mode
+		EditImg = new Image (Stock.Convert, IconSize.SmallToolbar);
+		
+		TabLabel.PackStart (EditImg, false, true, 2);
+		TabLabel.PackStart (titleLabel, true, true, 0);
+		TabLabel.PackStart (tabClose, false, false, 2);
+		
+		// needed, otherwise even calling show_all on the notebook won't
+		// make the hbox contents appear.
+		TabLabel.ShowAll();
+		EditImg.Visible = false;
+	
+	}
+	
+	public Mode TabMode {
+		get { return Tab_mode; }
+		set { Tab_mode = value; }
+	}
+
+	
+	public void SetMode (Mode m)
+	{
+		if (Tab_mode == m)
+			return;
+		
+		if (m == Mode.Viewer) {
+			this.Page = 0;
+			browser.paste1.Sensitive = false;
+			EditImg.Visible = false;
+		} else {
+			this.Page = 1;
+			browser.paste1.Sensitive = true;
+			EditImg.Visible = true;
+		}
+
+		Tab_mode = m;
+	}
+	
+	// Events for the Editor
+	
+	void OnInsertParaClicked (object sender, EventArgs a)
+	{
+		text_editor.Buffer.InsertAtCursor ("\n<para>\n</para>");
+	}
+
+	void OnInsertNoteClicked (object sender, EventArgs a)
+	{
+		text_editor.Buffer.InsertAtCursor ("\n<block subset=\"none\" type=\"note\">\n  <para>\n  </para>\n</block>");
+	}
+
+	void OnInsertExampleClicked (object sender, EventArgs a)
+	{
+		text_editor.Buffer.InsertAtCursor ("\n<example>\n  <code lang=\"C#\">\n  </code>\n</example>");
+	}
+
+	void OnInsertListClicked  (object sender, EventArgs a)
+	{
+		text_editor.Buffer.InsertAtCursor ("\n<list type=\"bullet\">\n  <item>\n    <term>First Item</term>\n  </item>\n</list>");
+
+	}
+
+	void OnInsertTableClicked (object sender, EventArgs a)
+	{
+		text_editor.Buffer.InsertAtCursor ("\n<list type=\"table\">\n  <listheader>\n    <term>Column</term>\n" +
+						   "    <description>Description</description>\n" +
+						   "  </listheader>\n" +
+						   "  <item>\n" +
+						   "    <term>Term</term>\n" +
+						   "    <description>Description</description>\n" +
+						   "  </item>\n" +
+						   "</list>");
+	 }
+
+	void OnInsertType (object sender, EventArgs a)
+	{
+		text_editor.Buffer.InsertAtCursor ("<see cref=\"T:System.Object\"/>");
+	}
+	void OnSaveEdits (object sender, EventArgs a)
+	{
+		try {
+			edit_node.InnerXml = text_editor.Buffer.Text;
+		} catch (Exception e) {
+			browser.statusbar.Pop (browser.context_id);
+			browser.statusbar.Push (browser.context_id, e.Message);
+			return;
+		}
+		EditingUtils.SaveChange (edit_url, browser.help_tree, edit_node);
+		SetMode (Mode.Viewer);
+		history.ActivateCurrent ();
+	}
+
+	void OnCancelEdits (object sender, EventArgs a)
+	{
+		SetMode (Mode.Viewer);
+		history.ActivateCurrent ();
+	}
+	void EditedTextChanged (object sender, EventArgs args)
+	{
+		StringWriter sw = new StringWriter ();
+		XmlWriter w = new XmlTextWriter (sw);
+		
+		try {
+			w.WriteStartElement ("html");
+			w.WriteStartElement ("body");
+			
+			edit_node.InnerXml = text_editor.Buffer.Text;
+			EditingUtils.RenderEditPreview (edit_url, browser.help_tree, edit_node, w);
+			
+			w.WriteEndElement ();
+			w.WriteEndElement ();
+			
+			w.Flush ();
+		} catch (Exception e) {
+			browser.statusbar.Pop (browser.context_id);
+			browser.statusbar.Push (browser.context_id, e.Message);
+			return;
+		}
+		browser.statusbar.Pop (browser.context_id);
+		browser.statusbar.Push (browser.context_id, "XML OK");
+		Gtk.HTMLStream s = html_preview.Begin ("text/html");
+		s.Write (sw.ToString ());
+		html_preview.End (s, HTMLStreamStatus.Ok);
+	}
+	void OnTabClose (object sender, EventArgs a)
+	{
+		browser.tabs_nb.RemovePage(browser.tabs_nb.PageNum(this));
+		browser.tabs_nb.ShowTabs = (browser.tabs_nb.NPages > 1);
+	}
+	
+}
 }
