@@ -168,8 +168,11 @@ public class EcmaProvider : Provider {
 						XmlDocument nsSummaryFile = new XmlDocument ();
 						nsSummaryFile.Load (ns_summary_file);
 						
-						namespace_summaries [tn] = nsSummaryFile.SelectSingleNode ("Namespace/Docs/summary");
-						namespace_remarks [tn] = nsSummaryFile.SelectSingleNode ("Namespace/Docs/remarks");
+						XmlNode ns_summary = nsSummaryFile.SelectSingleNode ("Namespace/Docs/summary");
+						if (ns_summary != null && ns_summary.InnerText != "To be added." && ns_summary.InnerText != "") {
+							namespace_summaries [tn] = nsSummaryFile.SelectSingleNode ("Namespace/Docs/summary");
+							namespace_remarks [tn] = nsSummaryFile.SelectSingleNode ("Namespace/Docs/remarks");
+						}
 						
 					} else if (!namespace_summaries.ContainsKey (tn)) {
 						namespace_summaries [tn] = null;
@@ -765,21 +768,7 @@ public class EcmaHelpSource : HelpSource {
 			return String.Format ("/Type [@FullName = '{0}']/Docs/{1}", type_name, n.Name);
 		}
 		
-		if (n.Matches ("/Type/Members/Member [MemberType != 'Constructor' and MemberType != 'Method']/Docs/*")) {
-			string type_name = (string) n.Evaluate ("string (ancestor::Type/@FullName)");
-			string member_name = (string) n.Evaluate ("string (ancestor::Member/@MemberName)");
-			string param_name = (string) n.Evaluate ("string (@name)");
-			
-			if (param_name == null || param_name == "") {
-				return String.Format ("/Type [@FullName = '{0}']/Members/Member [@MemberName = '{1}']/Docs/{2}",
-					type_name, member_name, n.Name);
-			} else {
-				return String.Format ("/Type [@FullName = '{0}']/Members/Member [@MemberName = '{1}']/Docs/param [@name = '{2}']",
-					type_name, member_name, param_name);
-			}
-		}
-		
-		if (n.Matches ("/Type/Members/Member [MemberType = 'Constructor' or MemberType = 'Method']/Docs/*")) {
+		if (n.Matches ("/Type/Members/Member/Docs/*")) {
 			string type_name = (string) n.Evaluate ("string (ancestor::Type/@FullName)");
 			string member_name = (string) n.Evaluate ("string (ancestor::Member/@MemberName)");
 			string member_sig = (string) n.Evaluate ("string (ancestor::Member/MemberSignature [@Language='C#']/@Value)");
@@ -1028,8 +1017,8 @@ public class EcmaHelpSource : HelpSource {
 	// types and members have been implemented and whether there are any
 	// MonoTODO attributes left on them. 
 
-	readonly ExtensionObject ExtObject;
-	class ExtensionObject {
+	public readonly ExtensionObject ExtObject;
+	public class ExtensionObject {
 		readonly EcmaHelpSource hs;
 		
 		public ExtensionObject (EcmaHelpSource hs)
