@@ -17,6 +17,28 @@ public class History {
 	int pos = -1;
 	ArrayList history;
 	
+	public bool Active {
+		get 
+		{
+			return active;
+		}
+		set 
+		{
+			if(value) {
+				if (pos > 0)
+					back.Sensitive = true;
+				else
+					back.Sensitive = false;
+				if (pos+1 == history.Count)
+					forward.Sensitive = false;
+				else
+					forward.Sensitive = true;
+			}
+			active = value;
+		}
+	}
+	private bool active;
+	
 	public History (Gtk.Button back, Gtk.Button forward)
 	{
 		this.back = back;
@@ -54,6 +76,8 @@ public class History {
 
 	internal void BackClicked (object o, EventArgs args)
 	{
+		if (!active)
+			return;
 		if (pos < 1)
 			return;
 		pos--;
@@ -68,6 +92,8 @@ public class History {
 
 	internal void ForwardClicked (object o, EventArgs args)
 	{
+		if (!active)
+			return;
 		if (pos+1 == history.Count)
 			return;
 
