@@ -389,6 +389,11 @@ public class HelpSource {
 			zipFileWriteTime = DateTime.Now;
 		}
 	}
+	
+	public HelpSource() {
+		Tree = new Tree (this, "Blah", "Blah");
+		source_id = id++;
+	}
 
 	public DateTime ZipFileWriteTime {
 		get {
@@ -653,6 +658,8 @@ public abstract class Provider {
 public class RootTree : Tree {
 	string basedir;
 	
+	public static Hashtable ExtraHelpSources = new Hashtable();
+	
 	public const int MonodocVersion = 1;
 	
 	public static RootTree LoadTree ()
@@ -692,6 +699,12 @@ public class RootTree : Tree {
 		root.name_to_node ["root"] = root;
 		root.Populate (root, nodes);
 
+		Node third_party = root.LookupEntryPoint ("various");
+		if (third_party == null) {
+			Console.Error.WriteLine ("No 'various' doc node! Check monodoc.xml!");
+			third_party = root;
+		}
+
 		//
 		// Load the sources
 		//
@@ -709,6 +722,11 @@ public class RootTree : Tree {
 				Console.Error.WriteLine ("Error: Could not load source file {0}", file);
 				continue;
 			}
+
+			XmlNodeList extra_nodes = doc.SelectNodes ("/monodoc/node");
+			if (extra_nodes.Count > 0)
+				root.Populate (third_party, extra_nodes);
+
 			XmlNodeList sources = doc.SelectNodes ("/monodoc/source");
 			if (sources == null){
 				Console.Error.WriteLine ("Error: No <source> section found in the {0} file", file);
@@ -745,6 +763,14 @@ public class RootTree : Tree {
 						break;
 					}
 					break;
+				case "ecma-uncompiled":
+					try {
+						hs = new EcmaUncompiledHelpSource (basefilepath);
+					} catch (FileNotFoundException) {
+						Console.Error.WriteLine ("Error: did not find one of the files in sources/"+basefilepath);
+						break;
+					}
+					break;
 				case "monohb":
 					try {
 						hs = new MonoHBHelpSource(basefilepath, false);
@@ -807,7 +833,7 @@ public class RootTree : Tree {
 				root.help_sources.Add (hs);
 				root.name_to_hs [path] = hs;
 
-				Node parent = (Node) root.name_to_node [path];
+				Node parent = root.LookupEntryPoint (path);
 				if (parent == null){
 					Console.Error.WriteLine ("node `{0}' is not defined on the documentation map", path);
 					continue;
@@ -819,6 +845,16 @@ public class RootTree : Tree {
 			}
 		}
 		
+		foreach (string key in ExtraHelpSources.Keys) {
+			HelpSource hs = (HelpSource)ExtraHelpSources[key];
+			root.help_sources.Add (hs);
+			Node hsn = root.CreateNode(key, "extra-help-source-" + key);
+			hsn.EnsureNodes ();
+			foreach (Node n in hs.Tree.Nodes){
+				hsn.AddNode (n);
+			}
+		}
+
 		return root;
 	}
 
@@ -1128,6 +1164,8 @@ public class RootTree : Tree {
 		chmod (Path.Combine (root.basedir, "monodoc.index"), 0x1a4);
 		Console.WriteLine ("Documentation index updated");
 	}
+	
+	public ICollection HelpSources { get { return new ArrayList(help_sources); } }
 
 	[System.Runtime.InteropServices.DllImport ("libc")]
 	static extern int chmod (string filename, int mode);
